/*
 * Bundesliga is Copyright (c) 1997-1999 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "bl.h"
#include "blrcp.h"

struct {
  FieldPtr m1, m2;
} editMatch[10];

static RectangleType eRect;
static PointType ePt;
static CharPtr MItems[20];
static DateTimeType date;

/**
 * Edit Form
 * purpose: edit the fixture by drag and drop
 */
static void SetDatumLabel()
{
  ControlPtr ctlP = Id2Ptr(ID_FrmEditDatum);
  char *label = CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  DateToAscii(date.month, date.day, date.year, 
  	      sysPrefs.longDateFormat, label);
  CtlDrawControl(Id2Ptr(ID_FrmEditDatum));
}

void InitFormEdit()
{
  int i;

  for (i=0; i<10; i++) {
    Boolean usable = i<(prefs.NumTeams>>1);
    editMatch[i].m1 = Id2Ptr(ID_FrmEditMann+i);
    editMatch[i].m2 = Id2Ptr(ID_FrmEditMann+i+10);
    FldSetUsable(editMatch[i].m1, usable);
    FldSetUsable(editMatch[i].m2, usable);
  }
  if (prefs.NumTeams % 2 == 1) {
    FldSetUsable(editMatch[prefs.NumTeams>>1].m2, true);
  }
  if (prefs.NumTeams<19) {
    for (i=0; i<9; i++) {
      FrmMoveObject(ID_FrmEditMann+i,0,i);
      FrmMoveObject(ID_FrmEditMann+10+i,0,i);
    }
  }

  if (Datum[matchDay] == 0) {
    TimSecondsToDateTime(TimGetSeconds(), &date);
  } else {
    if (Datum[matchDay]<100000) /* preserve compatibility for v0.4 */
      Datum[matchDay] = (Datum[matchDay] - 1461) * 86400;
    TimSecondsToDateTime(Datum[matchDay], &date);
  }
  SetDatumLabel();
}

static void DrawFormEdit()
{
  int i, g, m1, m2;
  ULong mask = 0UL;
  for (i=0; i<(prefs.NumTeams>>1); i++) {
    g = programme[matchDay*10+i];
    FldSetTextPtr(editMatch[i].m1, M1vSpNr(g));
    FldSetTextPtr(editMatch[i].m2, M2vSpNr(g));
    m1 = g / 100 - 1;
    m2 = g % 100 - 1;
    /* remember which teams play this round */
    mask |= ( 1UL<<m1 ) | ( 1UL<<m2 );
  }
  if (prefs.NumTeams % 2 == 1) {
    for (i=0; i<prefs.NumTeams; i++) {
      /* set team that doesn't play this round */
      if ( (mask & (1UL<<i)) == 0 ) { 
	FldSetTextPtr(editMatch[prefs.NumTeams>>1].m2,team[i]);
	break;
      }
    }
  }
}

/*
 * swap contents of the two fields
 */
static void exchange(FieldPtr m1, FieldPtr m2)
{
  CharPtr s1, s2;
  s1 = FldGetTextPtr(m1);
  s2 = FldGetTextPtr(m2);
  FldSetTextPtr(m1, s2);
  FldSetTextPtr(m2, s1);
  FldDrawField(m1);
  FldDrawField(m2);
}

static FieldPtr getTeamUnder(FieldPtr f)
{
  FieldAttrType attr;
  FldGetAttributes(f, &attr);
  if (!attr.usable) /* is field used */
    return NULL;
  FldGetBounds(f, &eRect);
  return f;
}

static FieldPtr teamUnder(SWord x, SWord y)
{
  int div = (prefs.NumTeams==20)?11:12;

  if (y<30 || y>137 || (x>75 && x<80) || x>145)
    return NULL;

  y = (y-30) / div;
  if (x<80) { /* left half  */
    return getTeamUnder(editMatch[y].m1);
  }
  /* right half */
  return getTeamUnder(editMatch[y].m2);
}

static void dragTeam(SWord x, SWord y)
{
  WinInvertRectangle(&eRect, 2);
  RctOffsetRectangle(&eRect, x-ePt.x, y-ePt.y);
  WinInvertRectangle(&eRect, 2);
  ePt.x = x;
  ePt.y = y;
}

/*
 * return number of team in team table of argument "s"
 */
static int MNrvString(CharPtr s)
{
  int i;

  /*
   * I've only mangled pointers, so I can compare the pointers
   */
  for (i=0; i<prefs.NumTeams; i++) {
    if (team[i] == s)
      return i+1;
  }
  return 0;
}

/*
 * correct settings in fixture according to drag & drop actions before;
 * if necessary reset results - the user was warned about this!
 */
static void UpdateSpielplan()
{
  int i, m1, m2;
  for (i=0; i<(prefs.NumTeams>>1); i++) {
    m1 = MNrvString(FldGetTextPtr(editMatch[i].m1));
    m2 = MNrvString(FldGetTextPtr(editMatch[i].m2));
    if (m1 && m2) {
      programme[matchDay*10+i] = m1*100 + m2;
    }
  }
  for (i=0; i<prefs.NumTeams; i++) {
    MStatus[i].Erg[matchDay] = 0xffff;
  }
}

/*
 * are there any results entered for this match day?
 */
static Boolean FormHasResults()
{
  int i;
  for (i=0; i<prefs.NumTeams; i++) {
    if (MStatus[i].Erg[matchDay] != 0xffff) 
      return true;
  }
  return false;
}

Boolean EditFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static FieldPtr lastTeam;
  FieldPtr newTeam;
  
CALLBACK_PROLOGUE
  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    handled = true;
    if (FormHasResults()) {
      int but = FrmAlert(ID_FrmHasResults);
      if (but == 1) {
	/* don't delete results, but return to form immediately */
	FrmReturnToForm(0);
	FrmEraseForm(FrmGetActiveForm());
	DrawBegegnungen(true);
	break;
      }
    }
    DrawFormEdit();
    FrmDrawForm(FrmGetActiveForm());
    break;

  case penDownEvent:
    /* are we over a field with a team? */
    if ((lastTeam = teamUnder(e->screenX, e->screenY))) {
      WinInvertRectangle(&eRect, 2);
      ePt.x = e->screenX;
      ePt.y = e->screenY;
      handled = true;
    }
    break;

  case penMoveEvent:
    /* move rectangle */
    if (lastTeam) {
      dragTeam(e->screenX, e->screenY);
      handled = true;
    }
    break;

  case penUpEvent:
    /* swap teams */
    if (lastTeam) {
      WinInvertRectangle(&eRect, 2);
      if ((newTeam = teamUnder(e->screenX, e->screenY))) {
	exchange(lastTeam,newTeam);
	handled = true;
      }
    }
    break;

  case ctlSelectEvent:
    /* "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton) {
      UpdateSpielplan();
      Datum[matchDay] = TimDateTimeToSeconds(&date);
      FrmReturnToForm(0);
      FrmEraseForm(FrmGetActiveForm());
      DrawBegegnungen(true);
      handled = true; 
      break;
    }

    /* "Cancel" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+1) {
      FrmReturnToForm(0);
      FrmEraseForm(FrmGetActiveForm());
      DrawBegegnungen(true);
      handled = true; 
      break;
    }

    /* "Date" */
    if (e->data.ctlSelect.controlID == ID_FrmEditDatum) {
      if ( SelectDay(&date.month, &date.day, &date.year, CHOOSE_DATE) ) {
	SetDatumLabel();
      }
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/**
 * routines handling editing and entering teams list
 */
void InitFormTeams()
{
  SelectedDBInfo info;
  ListPtr p;
  int i;

  SelectedDB(DBNr, &info);
  FrmCopyLabel(FrmGetActiveForm(), ID_FrmTeamsName, info.dbName);

  for (i=0; i<prefs.NumTeams; i++) {
    MItems[i] = team[i];
  }

  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  LstSetListChoices(p, MItems, prefs.NumTeams);
  LstSetHeight(p, 10);
}

static int TeamsListGetSelection()
{
  ListPtr p;
  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  return LstGetSelection(p);
}

static void RedrawList()
{
  ListPtr p;
  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  LstEraseList(p);
  LstDrawList(p);
}

Boolean TeamsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Button "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton) {
      FrmReturnToForm(0);
      DrawBegegnungen(true);
      handled = true; 
      break;
    }

    /* Button "Edit" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton+1) {
      FrmPopupForm(ID_FrmEnterMann);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

static void DrawFormEnterMann()
{
  int nr = TeamsListGetSelection();
  CharPtr  name;
  FieldPtr fld;
  VoidHand handle;

  if (nr == -1)
    return;

  fld = Id2Ptr(ID_FrmEnterMannFld);
  if (!(handle = (VoidHand)FldGetTextHandle(fld))) {
    handle = MemHandleNew(18*sizeof(char));
    name = MemHandleLock(handle);
    StrCopy(name, team[nr]);
    MemHandleUnlock(handle);
    FldSetTextHandle(fld, (Handle)handle);
  } else {
    name = MemHandleLock(handle);
    StrCopy(name, team[nr]);
    MemHandleUnlock(handle);
  }
  
  FldSetUsable(fld, true);
  FldDrawField(fld);
}

static void SetEnteredMann()
{
  int nr = TeamsListGetSelection();
  StrCopy(team[nr], FldGetTextPtr(Id2Ptr(ID_FrmEnterMann)));
}

Boolean EnterMannFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    DrawFormEnterMann();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    if (e->data.ctlSelect.controlID == ID_FrmEnterMannButton) {
      SetEnteredMann();
      FrmReturnToForm(0);
      RedrawList();
      handled = true; 
      break;
    }

    if (e->data.ctlSelect.controlID == ID_FrmEnterMannButton+1) {
      FrmReturnToForm(0);
      RedrawList();
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

/**
 * routines handling creation of a new database
 */
static void InitFrmNewDB()
{
  FieldPtr fld;
  VoidHand handle;
  CharPtr name;
  int i, s[3] = { 19, 3, 3 };

  for (i=0; i<2; i++) {
    fld = Id2Ptr(ID_FrmNewDBFld);
    if (!(handle = (VoidHand)FldGetTextHandle(fld))) {
      handle = MemHandleNew(s[i]*sizeof(char));
      name = MemHandleLock(handle);
      StrCopy(name, "");
      MemHandleUnlock(handle);
      FldSetTextHandle(fld, (Handle)handle);
    } else {
      name = MemHandleLock(handle);
      StrCopy(name, "");
      MemHandleUnlock(handle);
    }
  }
}

/*
 * create database according to entries in form
 */
static void ReadFormValues()
{
  CharPtr name, nr, rds;
  DmOpenRef db;
  BlDatabasePreferencesType newPrefs;
  int n = 0, m = 0;

  nr = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+1));
  rds = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+2));

  if (nr)  n = StrAToI(nr);
  if (rds) m = StrAToI(rds);

  if (n < 3 || n > 20 || m < n-1 || m > 38) {
    FrmAlert(ID_FrmAlertNumTeams);
    return;
  }

  name = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld));

  newPrefs.NumTeams  = n;
  newPrefs.Spieltage = m;
  newPrefs.matchDay  = 0;
  newPrefs.Version   = 3;
  
  if ( name && (db = CreateDatabase(name, &newPrefs)) ) {
    InitNewDatabase(db);
    DmCloseDatabase(db);
  }
}

Boolean NewDBFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFrmNewDB();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    if (e->data.ctlSelect.controlID == ID_FrmNewDBButton) {
      ReadFormValues();
      FrmGotoForm(ID_FrmBl);
      handled = true; 
      break;
    }

    if (e->data.ctlSelect.controlID == ID_FrmNewDBButton+1) {
      FrmReturnToForm(0);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

static void InitFrmRenameLeague()
{
  FieldPtr fld;
  VoidHand handle;
  CharPtr name;
  SelectedDBInfo info;

  SelectedDB(DBNr, &info);

  fld = Id2Ptr(ID_FrmRenameLeagueFld);
  if (!(handle = (VoidHand)FldGetTextHandle(fld))) {
    handle = MemHandleNew(26*sizeof(char));
    name = MemHandleLock(handle);
    StrCopy(name, info.dbName);
    MemHandleUnlock(handle);
    FldSetTextHandle(fld, (Handle)handle);
  } else {
    name = MemHandleLock(handle);
    StrCopy(name, info.dbName);
    MemHandleUnlock(handle);
  }
}

static void RenameDatabaseFile()
{
  CharPtr name = FldGetTextPtr(Id2Ptr(ID_FrmRenameLeagueFld));
  if (name) {
    RenameDatabase(DBNr, name);
  }
}

Boolean RenameLeagueFormHandleEvent(EventPtr e)
{
  Boolean handled;

CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    InitFrmRenameLeague();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    if (e->data.ctlSelect.controlID == ID_FrmRenameLeagueButton) {
      RenameDatabaseFile();
      FrmReturnToForm(0);
      FrmEraseForm(FrmGetActiveForm());
      DrawBegegnungen(true);
      handled = true; 
      break;
    }

    if (e->data.ctlSelect.controlID == ID_FrmRenameLeagueButton+1) {
      FrmReturnToForm(0);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}

Boolean SettingsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  FormPtr frm;
  static Byte editMode;

CALLBACK_PROLOGUE

  handled = false;
  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsButton + 2), 
		       appPrefs.editMode & 1);
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsButton + 3), 
		       appPrefs.editMode & 2);
    editMode = appPrefs.editMode;
    FrmDrawForm(frm);
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmSettingsButton:
      appPrefs.editMode = editMode; 

    case ID_FrmSettingsButton+1:
      FrmReturnToForm(0);
      handled = true; 
      break;

    case ID_FrmSettingsButton+2:
      handled = true; 
      editMode ^= 1;
      break;

    case ID_FrmSettingsButton+3:
      handled = true; 
      editMode ^= 2;
      break;
    }
    break;

  default:
    break;
  }

CALLBACK_EPILOGUE
  return handled;
}
